from deampy.plots.eff_sample_size import *

plot_eff_sample_size(likelihood_weights=[0.1, 0.2, 0.3, 0.4], if_randomize=False)
plot_eff_sample_size(likelihood_weights=[0.1, 0.2, 0.3, 0.4], if_randomize=True)
