from tests.prob_dists.fit_dist_tests import *

# simulate some data
np.random.seed(1)
test_fitting_beta()
test_fitting_beta_binomial()
test_fitting_binomial()
test_fitting_empirical()
test_fitting_exponential()
test_fitting_gamma()
test_fitting_gamma_poisson()
test_fitting_geometric()
test_fitting_johnson_sb()
test_fitting_johnson_su()
test_fitting_lognormal()
test_fitting_negbinomial()
test_fitting_normal()
test_fitting_poisson()
test_fitting_triangular()
test_fitting_uniform()
test_fitting_uniform_discrete()
test_fitting_weibull()


